﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.Algorithms.PushGP.Stack;

  public abstract class RotateExpression<T> : StatelessExpression
  {
    public void Eval(IStack<T> stack)
    {
      if (stack.Count < 3) return;

      stack.Swap(3);
    }
  }

  public class IntegerRotateExpression : RotateExpression<long>
  {
    protected override string InitStringRepresentation()
    {
      return "INTEGER.ROT";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.IntegerStack);
    }
  }

  public class FloatRotateExpression : RotateExpression<double>
  {
    protected override string InitStringRepresentation()
    {
      return "FLOAT.ROT";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.FloatStack);
    }
  }

  public class BooleanRotateExpression : RotateExpression<bool>
  {
    protected override string InitStringRepresentation()
    {
      return "BOOLEAN.ROT";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.BooleanStack);
    }
  }

  public class NameRotateExpression : RotateExpression<string>
  {
    protected override string InitStringRepresentation()
    {
      return "NAME.ROT";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.NameStack);
    }
  }

  public class ExecRotateExpression : RotateExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "EXEC.ROT";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.ExecStack);
    }
  }

  public class CodeRotateExpression : RotateExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "CODE.ROT";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.CodeStack);
    }
  }
}