﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.Algorithms.PushGP.Stack;

  /// <summary>
  ///     Swaps the top two values.
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class SwapExpression<T> : StatelessExpression
  {
    public void Eval(IStack<T> stack)
    {
      if (stack.Count < 2) return;

      stack.Swap(2);
    }
  }

  public class IntegerSwapExpression : SwapExpression<long>
  {
    protected override string InitStringRepresentation()
    {
      return "INTEGER.SWAP";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.IntegerStack);
    }
  }

  public class FloatSwapExpression : SwapExpression<double>
  {
    protected override string InitStringRepresentation()
    {
      return "FLOAT.SWAP";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.FloatStack);
    }
  }

  public class BooleanSwapExpression : SwapExpression<bool>
  {
    protected override string InitStringRepresentation()
    {
      return "BOOLEAN.SWAP";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.BooleanStack);
    }
  }

  public class NameSwapExpression : SwapExpression<string>
  {
    protected override string InitStringRepresentation()
    {
      return "NAME.SWAP";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.NameStack);
    }
  }

  public class ExecSwapExpression : SwapExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "EXEC.SWAP";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.ExecStack);
    }
  }

  public class CodeSwapExpression : SwapExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "CODE.SWAP";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.CodeStack);
    }
  }
}